/***************************************************************************
 *   Copyright (C) 2006 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#include "DebugLog.h"
#include "GuitoneCore.h"
#include "GuitoneStandalone.h"
#include "GuitoneDriver.h"

#include <QLocale>
#include <QTranslator>
#include <QLibraryInfo>

void guitoneMsgHandler(QtMsgType type, const char *msg)
{
    switch (type)
    {
        case QtDebugMsg:
            DebugLog::debug(msg);
            break;
        case QtWarningMsg:
            DebugLog::warn(msg);
            break;
        case QtCriticalMsg:
            DebugLog::critical(msg);
            break;
        case QtFatalMsg:
            DebugLog::fatal(msg);
            abort();
     }
}

int main(int argc, char** argv)
{
    // install our own message handler to catch debug messages, warnings, etc.
    qInstallMsgHandler(guitoneMsgHandler);

    QString locale = QLocale::system().name();

    // load the default Qt translations
    QTranslator qtTranslator;
    QString transPath = QLibraryInfo::location(QLibraryInfo::TranslationsPath);
    if (!qtTranslator.load("qt_" + locale, transPath) && !locale.startsWith("en"))
    {
        W(QString("Couldn't load qt translation for %1 - using default")
          .arg(locale));
    }

    // try to load the application's translation
    QTranslator appTranslator;
    QString transFileName("guitone_" + locale);

    if (!appTranslator.load(transFileName, ":/i18n") && !locale.startsWith("en"))
    {
        W(QString("Couldn't load app translation for %1 - using default")
          .arg(locale));
    }

    GuitoneCore * guitone;
    if (argc > 1 && strcmp(argv[1], "exec") == 0)
    {
        guitone = new GuitoneDriver(argc, argv);
    }
    else
    {
        guitone = new GuitoneStandalone(argc, argv);
    }

    guitone->installTranslator(&appTranslator);
    guitone->installTranslator(&qtTranslator);

    int retValue = guitone->exec();
    delete guitone;

    return retValue;
}

