/***************************************************************************
 *   Copyright (C) 2007 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#ifndef MONOTONE_MANAGER_H
#define MONOTONE_MANAGER_H

#include "MonotoneThread.h"

#include <QMultiMap>

class MonotoneManager : public QObject
{
    Q_OBJECT
public:
    MonotoneManager();
    ~MonotoneManager();

    //! set the path to the monotone binary to use for all new threads
    bool setMonotoneBinaryPath(const QString &, bool, QString &);

    //! returns if the path to the monotone binary was set successful before
    inline bool monotoneBinaryPathSet() const { return !mtnPath.isEmpty(); }

    //! returns the database filepath for a given workspace path
    DatabaseFile getDatabaseFilePath(const WorkspacePath &);

    //! returns an appropriate MonotoneThread for the given workspace
    MonotoneThread * getThreadForWorkspace(const WorkspacePath &);

    //! returns an appropriate MonotoneThread for the given database
    MonotoneThread * getThreadForDatabase(const DatabaseFile &);

    //! returns a thread by a given thread number
    MonotoneThread * getThreadByNumber(int);

    //! runs a single command and returns its output
    static bool singleRun(const QString &, const QStringList &, QByteArray &);

    //! returns the interface version of a given monotone binary
    static QString getInterfaceVersion(const QString &);

    //! returns the directory which contains the _MTN directory
    static WorkspacePath normalizeWorkspacePath(const WorkspacePath &);

    //! compares two version strings
    static int versionCompare(const QString &, const QString &);

    //! the minimum required interface version needed
    static const QString MinInterfaceVersion;
    //! the maximum allowed interface version (excluding this particular version)
    static const QString MaxInterfaceVersion;

signals:
    //! is emitted if a thread is aborted and contains some diagnostic message
    void threadAborted(const QString &);

private:
    MonotoneThread * getThread(const DatabaseFile &, const WorkspacePath &);

    QMap<int, MonotoneThread *> threadMap;
    QMultiMap<int, QString> identMap;
    QMap<WorkspacePath, DatabaseFile> workspaceMap;

    QString mtnPath;
    int threadNumber;
    QMutex lock;

private slots:
    void aborted(int, QProcess::ProcessError, const QString &);
};

#endif

