/***************************************************************************
 *   Copyright (C) 2007 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#include "StdioParser.h"
#include "vocab.h"

StdioParser::StdioParser(const QByteArray & input) : AbstractParser(input) {}

bool StdioParser::parse()
{
    if (getLeftBytesCount() == 0) return false;

    // chunk format: <cmdNumber>:<errCode>:<chunkType>:<chunkSize>:<payload>
    commandNumber = getNumber();
    I(getNext() == ':');
    errorCode = getNumber();
    I(getNext() == ':');
    chunkType = getNext();
    I(chunkType == 'm' || chunkType == 'l');
    I(getNext() == ':');
    chunkSize = getNumber();
    I(getNext() == ':');

    int charsLeft = getLeftBytesCount();
    if (chunkSize > charsLeft)
    {
        return false;
    }

    payload = getNext(chunkSize);
    return true;
}

int StdioParser::getNumber()
{
    int number = 0;
    int processedChars = 0;

    while (true)
    {
        char ch = whatsNext();
        if (ch < '0' || ch > '9')
        {
            // ensure that we've read at least one char
            I(processedChars > 0);
            break;
        }
        number *= 10;
        number += (ch - '0');
        advance();
        processedChars++;
    }

    return number;
}

