/***************************************************************************
 *   Copyright (C) 2007 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#include "MainWindow.h"
#include "WorkspaceMenuBar.h"
#include "DatabaseMenuBar.h"
#include "Settings.h"
#include "GuitoneStandalone.h"

#include <QCloseEvent>
#include <QDesktopWidget>

MainWindow::MainWindow() : QMainWindow(), menuBar(0), dialogManager(0)
{
    setObjectName("MainWindow");
}

MainWindow::~MainWindow()
{
    if (menuBar)        delete menuBar;
    if (dialogManager)  delete dialogManager;
}

void MainWindow::init()
{
    if (!menuBar)
    {
        menuBar = new MenuBar(this);
    }

    if (!dialogManager)
    {
        dialogManager = new DialogManager(this);
    }

    connect(
        dialogManager, SIGNAL(allDialogsClosed()),
        this, SLOT(allDialogsClosed())
    );

    connect(
        menuBar, SIGNAL(showAbout()),
        dialogManager, SLOT(showAbout())
    );

    connect(
        menuBar, SIGNAL(showPreferences()),
        dialogManager, SLOT(showPreferences())
    );

    connect(
        menuBar, SIGNAL(closeWindow()),
        this, SLOT(close())
    );

    GuitoneStandalone * app = reinterpret_cast<GuitoneStandalone *>(APP);

    // since the window is deleted afterwards, make this connection
    // queued so the slot call does not return to invalid memory
    connect(
        this, SIGNAL(windowClosed(MainWindow *)),
        app, SLOT(windowClosed(MainWindow *)),
        Qt::QueuedConnection
    );

    connect(
        menuBar, SIGNAL(loadWorkspace(const QString &)),
        app, SLOT(loadWorkspace(const QString &))
    );

    connect(
        menuBar, SIGNAL(loadDatabase(const QString &)),
        app, SLOT(loadDatabase(const QString &))
    );

    connect(
        menuBar, SIGNAL(checkForApplicationUpdates()),
        app, SLOT(checkForApplicationUpdates())
    );

    connect(
        menuBar, SIGNAL(activateWindow(int)),
        app, SLOT(activateWindow(int))
    );

    connect(
        menuBar, SIGNAL(raiseAllWindows()),
        app, SLOT(raiseAllWindows())
    );

    connect(
        app, SIGNAL(updateRecentLists()),
        menuBar, SLOT(updateRecentLists())
    );

    connect(
        app, SIGNAL(updateWindowList(const QStringList &)),
        menuBar, SLOT(updateWindowList(const QStringList &))
    );

    // common layout tasks
    setMenuBar(menuBar);
    restoreGeometry(Settings::getWindowGeometry(objectName()));

    setup();
}

void MainWindow::closeEvent(QCloseEvent * event)
{
    event->accept();

    Settings::setWindowGeometry(objectName(), saveGeometry());
    // FIXME: what if certain dialogs do not want to be closed?
    dialogManager->closeAllDialogs();
    emit windowClosed(this);
}

void MainWindow::allDialogsClosed()
{
    raise();
    activateWindow();
}
