/***************************************************************************
 *   Copyright (C) 2007 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#ifndef WORKSPACEWINDOW_H
#define WORKSPACEWINDOW_H

#include "DatabaseWindow.h"

#include "InventoryModel.h"
#include "InventoryProxyModel.h"
#include "InventoryWatcher.h"
#include "GetAttributes.h"

#include "InventoryView.h"
#include "AttributesView.h"
#include "Splitter.h"
#include "IconHelp.h"
#include "NodeInfo.h"

#include <QStatusBar>

class WorkspaceWindow: public DatabaseWindow
{
    Q_OBJECT
public:
    WorkspaceWindow();
    ~WorkspaceWindow();

    virtual void init();
    virtual void setup();
    //! throws GuitoneException
    virtual void load(const QString &);

protected:
    WorkspacePath workspacePath;

    Splitter * mainSplitter;
    Splitter * listSplitter;
    InventoryView  * treeView;
    InventoryView  * listView;
    AttributesView * attrView;
    QStatusBar * statusBar;
    IconHelp * iconHelp;
    NodeInfo * nodeInfo;

    InventoryModel      * invModel;
    GetAttributes       * attrModel;
    InventoryProxyModel * proxyModelFolderTree;
    InventoryProxyModel * proxyModelFileList;
    InventoryWatcher    * invWatcher;

private slots:
    void maybeReadNodeInfo(const QModelIndexList &);
    void invalidWorkspaceFormat(const QString &);
    void openFile(const QString &);
};

#endif
