/***************************************************************************
 *   Copyright (C) 2007 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#include "OpenPrompt.h"
#include "Settings.h"

#include <QFileDialog>

OpenPrompt::OpenPrompt(QWidget * parent) : Dialog(parent)
{
    setupUi(this);
    Dialog::init();

    connect(
        btnOpenDatabase, SIGNAL(clicked()),
        this, SLOT(openDatabase())
    );

    connect(
        btnOpenWorkspace, SIGNAL(clicked()),
        this, SLOT(openWorkspace())
    );

    connect(
        btnExit, SIGNAL(clicked()),
        this, SIGNAL(quitApp())
    );

    connect(
        this, SIGNAL(rejected()),
        this, SIGNAL(quitApp())
    );

    QStringList workspaces = Settings::getItemList("RecentWorkspaceList");

    if (workspaces.size() > 0)
    {
        recentWorkspaces->addItems(workspaces);
        recentWorkspaces->setEnabled(true);

        connect(
            recentWorkspaces, SIGNAL(currentIndexChanged(int)),
            this, SLOT(recentWorkspaceChanged(int))
        );
    }
    QStringList databases = Settings::getItemList("RecentDatabaseList");

    if (databases.size() > 0)
    {
        recentDatabases->addItems(databases);
        recentDatabases->setEnabled(true);

        connect(
            recentDatabases, SIGNAL(currentIndexChanged(int)),
            this, SLOT(recentDatabaseChanged(int))
        );
    }

    dialogManager = new DialogManager(this);

    connect(
        btnAbout, SIGNAL(clicked()),
        dialogManager, SLOT(showAbout())
    );

    connect(
        btnPreferences, SIGNAL(clicked()),
        dialogManager, SLOT(showPreferences())
    );
}

OpenPrompt::~OpenPrompt() {};

void OpenPrompt::openDatabase()
{
    QString fn = QFileDialog::getOpenFileName(
        this,
        tr("Select your database..."),
        QString(),
        tr("monotone Databases (*.mtn *.db)")
    );

    if (fn.isEmpty()) return;
    setEnabled(false);
    setDisabled(true);
    emit loadDatabase(fn);
}

void OpenPrompt::openWorkspace()
{
    QString fn = QFileDialog::getExistingDirectory(
        this,
        tr("Select your workspace...")
    );

    if (fn.isEmpty()) return;

    setDisabled(true);
    emit loadWorkspace(fn);
}

void OpenPrompt::recentWorkspaceChanged(int idx)
{
    // skip the first dummy item
    if (idx < 1) return;
    setDisabled(true);
    emit loadWorkspace(recentWorkspaces->itemText(idx));
}

void OpenPrompt::recentDatabaseChanged(int idx)
{
    // skip the first dummy item
    if (idx < 1) return;
    setDisabled(true);
    emit loadDatabase(recentDatabases->itemText(idx));
}

