/***************************************************************************
 *   Copyright (C) 2008 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#include "NodeInfo.h"

#include <QFileInfo>
#include <QFileIconProvider>
#include <QDateTime>

NodeInfo::NodeInfo(QWidget * parent) : QDockWidget(parent)
{
    setVisible(false);

    QWidget * parentWidget = new QWidget(this);
    setupUi(parentWidget);
    setWidget(parentWidget);

    connect(
        fileBirthRev, SIGNAL(linkActivated(const QString &)),
        this, SIGNAL(showSelectRevision(const QString &))
    );

    resetInfo();

    setWindowTitle(tr("Node information"));
    setFloating(true);
    setAllowedAreas(Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
}

NodeInfo::~NodeInfo() {}

void NodeInfo::setWorkspacePath(const WorkspacePath & wp)
{
    workspacePath = wp;
}

void NodeInfo::readAndSetInfo(const InventoryItem * item)
{
    I(item);
    I(!workspacePath.isEmpty());

    QString strFileSize("-");
    QString strFileLastModified("-");

    IconProvider * ourIconProvider = IconProvider::singleton();
    if (item->isDirectory())
    {
        fileIcon->setPixmap(ourIconProvider->getPlainFolderIcon().pixmap(64, 64));
    }
    else
    {
        if (item->getFSType() == InventoryItem::File)
        {
            QFileInfo fileInfo(workspacePath + "/" + item->getPath());
            I(fileInfo.exists());
            strFileSize = QString("%1 KB").arg(fileInfo.size() / 1024);
            strFileLastModified = fileInfo.lastModified().toString(Qt::DefaultLocaleShortDate);
            QFileIconProvider iconProvider;
            fileIcon->setPixmap(iconProvider.icon(fileInfo).pixmap(64, 64));
        }
        else
        {
            fileIcon->setPixmap(ourIconProvider->getPlainFileIcon().pixmap(64, 64));
        }
    }

    filePath->setText(tr("Current Path: %1").arg(item->getPath()));
    fileStatus->setText(tr("Status: %1").arg(item->getStatusString()));

    QString renameSource = item->getRenameSource();
    if (renameSource.isEmpty()) renameSource = "-";
    fileOldPath->setText(tr("Old Path: %1").arg(renameSource));

    QString renameTarget = item->getRenameTarget();
    if (renameTarget.isEmpty()) renameTarget = "-";
    fileNewPath->setText(tr("New Path: %1").arg(renameTarget));

    QString birthRev = item->getBirthRevision();
    if (!birthRev.isEmpty())
    {
        fileBirthRev->setText(tr("Added in: <a href=\"%1\">%2...</a>").
                              arg(birthRev).
                              arg(birthRev.left(12)));
    }
    else
    {
        fileBirthRev->setText(tr("Added in: %1").arg("-"));
    }

    fileSize->setText(tr("%1").arg(strFileSize));
    fileBaseType->setText(tr("Old Node Type: %1").
                          arg(InventoryItem::translateFileType(item->getOldType())));
    fileNewType->setText(tr("New Node Type: %1").
                         arg(InventoryItem::translateFileType(item->getNewType())));
    fileDiskType->setText(tr("Current Node Type: %1").
                          arg(InventoryItem::translateFileType(item->getFSType())));
    fileLastModified->setText(tr("Last Modified: %1").arg(strFileLastModified));
}

void NodeInfo::resetInfo()
{
    IconProvider * ourIconProvider = IconProvider::singleton();
    fileIcon->setPixmap(ourIconProvider->getPlainFileIcon().pixmap(64, 64));

    filePath->setText(tr("Current Path: %1").arg("-"));
    fileStatus->setText(tr("Status: %1").arg("-"));
    fileOldPath->setText(tr("Old Path: %1").arg("-"));
    fileNewPath->setText(tr("New Path: %1").arg("-"));
    fileBirthRev->setText(tr("Added in: %1").arg("-"));
    fileSize->setText(tr("%1").arg("-"));
    fileBaseType->setText(tr("Old Node Type: %1").arg("-"));
    fileNewType->setText(tr("New Node Type: %1").arg("-"));
    fileDiskType->setText(tr("Current Node Type: %1").arg("-"));
    fileLastModified->setText(tr("Last Modified: %1").arg("-"));
}

